from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseProviderCheck


class {{title}}(BaseResourceCheck):
    def __init__(self):
        # This is the full description of your check
        description = "{{desc}}"

        # This is the Unique ID for your check
        id = "CKV_{{provider}}_{{index}}"

        # These are the terraform objects supported by this check (ex: aws_iam_policy_document)
        supported_provider = ['{{supported_resource}}']

        # Valid CheckCategories are defined in checkov/common/models/enums.py
        categories = [CheckCategories.{{category}}]
        super().__init__(name=description, id=id, categories=categories, supported_provider=supported_provider)

    def scan_provider_conf(self, conf):
        # Replace this with the custom logic for your check
        if 'policy' in conf.keys():
            return CheckResult.FAILED

        return CheckResult.PASSED


check = {{title}}()
